%
%
%  Intensity Profile Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%         <Submenu name="Spots Functions">        
%        <Item name="Intensity Profile" icon="Matlab" tooltip="Plot intensity between two selected spots, Graph intensity in Time, and display position of intensity">
%          <Command>MatlabXT::XTIntensityProfile(%i)</Command>
%        </Item>
%         </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Intensity Profile" icon="Matlab" tooltip="Plot intensity between two selected spots, Graph intensity in Time, and display position of intensity">
%            <Command>MatlabXT::XTIntensityProfile(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
%%
%  Description:
%   
%   Shows intensity profile along a line in time.
%   To use this XTension you need to create and select a spots object:
%     The line for the intensity profile consists in the space between 
%     a sequence of points in a spots object. 
%   To create such spots: Open Imaris, create a new spots component in
%     the surpass scene, press cancel (creation wizard), go to edit mode
%     and manually create points at the extends of the segments to profile.
%   The order of the points is important.
%   A second spots object is created by the XTension to show the extends 
%     of the segments and the point where the intensity is highlighted
%     in the plot at every time point; this spots object is erased when 
%     the GUI of the XTension is closed. 
%   Changes of the spots positions and the selected time point on the user 
%     interface of the XTension are reflected in Imaris. 
%   Press refresh when datset is changed or source spots are moved
%     (utility spots are repositioned and the intensity profile is 
%     recalculated).
%   
%

function XTIntensityProfile(aImarisApplicationID)

if isempty(cell2mat(strfind(javaclasspath('-all'), 'ImarisLib.jar')))
  mlock
  javaaddpath ImarisLib.jar
  munlock
end

% persistent variables for callbacks 
persistent mImarisApplication mUIControls mState mValues

% button callbacks
if isa(aImarisApplicationID, 'numeric') && aImarisApplicationID < 0
  if aImarisApplicationID == -1
    if ~isempty(mState.mSpots)
      mImarisApplication.GetSurpassScene.RemoveChild(mState.mSpots)
    end
    stop(mUIControls.mTimer)
    clear mImarisApplication mUIControls mState mValues
  else
    vPlaying = mState.mPlaying;
    if aImarisApplicationID ~= -12 && aImarisApplicationID ~= -9
      mState.mPlaying = false;
    end
    [mState, mValues] = Process(-aImarisApplicationID, ...
      mImarisApplication, mUIControls, mState, mValues);
    if ~vPlaying && mState.mPlaying
      start(mUIControls.mTimer)
    elseif vPlaying && ~mState.mPlaying
      stop(mUIControls.mTimer)
      RefreshPlot(mImarisApplication, mUIControls, mState, mValues)
    end
  end
  return
end % button callbacks

% get the application object
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  % called from workspace
  mImarisApplication = aImarisApplicationID;
else
  % connect to Imaris interface
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  mImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

mState.mTime = mImarisApplication.GetVisibleIndexT + 1;
mState.mPosition = 1;
mState.mShow = 0; % 0 - position, 1 - time
mState.mChannelNames = {};
mState.mChannelColors = [];
mState.mSpots = [];
mState.mPlaying = false;
mState.mInterval = 0.2;
mState.mUpdateImaris = true;
mState.mMinMax = [0, 0];

mUIControls = BuildGUI();

mValues = [];

XTIntensityProfile(-2); % resize
XTIntensityProfile(-7); % refresh


%-------------------------------------------------------------------------%
%% GUI Callbacks
%-------------------------------------------------------------------------%
function [aState, aValues] = Process(aEventNumber, ...
  aImarisApplication, aUIControls, aState, aValues)

% which action has been requested? 
switch aEventNumber
  case 2 % resize window
    ResizeWindow(aUIControls);
  case 3 % position changed
    aState.mPosition = round(get(aUIControls.mSliderPosition, 'Value'));
    set(aUIControls.mSliderPosition, 'Value', aState.mPosition);
    RefreshPlot(aImarisApplication, aUIControls, aState, aValues)
  case 4 % time changed
    aState.mTime = round(get(aUIControls.mSliderTime, 'Value'));
    set(aUIControls.mSliderTime, 'Value', aState.mTime);
    aImarisApplication.SetVisibleIndexT(aState.mTime - 1);
    RefreshPlot(aImarisApplication, aUIControls, aState, aValues)
  case 5 % toggle changed
    set(aUIControls.mCheckPosition, 'Value', 1);
    set(aUIControls.mCheckTime, 'Value', 0);
    aState.mShow = 0;
    RefreshPlot(aImarisApplication, aUIControls, aState, aValues)
  case 6 % toggle changed
    set(aUIControls.mCheckPosition, 'Value', 0);
    set(aUIControls.mCheckTime, 'Value', 1);
    aState.mShow = 1;
    RefreshPlot(aImarisApplication, aUIControls, aState, aValues)
  case 7 % refresh
    [aValues, aState] = RefreshValues(aImarisApplication, aUIControls, aState);
    RefreshPlot(aImarisApplication, aUIControls, aState, aValues)    
  case 8 % export
    ExportValues(aImarisApplication, aState, aValues);
  case 9 % play
    aState.mPlaying = ~aState.mPlaying;
  case 10 % speed
    vInterval = str2double(get(aUIControls.mEditInterval, 'String'));
    if ~isnan(vInterval)
      aState.mInterval = min(max(vInterval, 0.002), 1);
      set(aUIControls.mTimer, 'Period', aState.mInterval);
    end
    set(aUIControls.mEditInterval, 'String', num2str(aState.mInterval))
  case 11 % update
    aState.mUpdateImaris = get(aUIControls.mCheckUpdateImaris, 'Value');
  case 12 % timer
    if aState.mShow == 0 % position
      aState.mTime = aState.mTime + 1;
      if aState.mTime > size(aValues, 2)
        aState.mTime = 1;
      end
      set(aUIControls.mSliderTime, 'Value', aState.mTime);
    else
      aState.mPosition = aState.mPosition + 1;
      if aState.mPosition > size(aValues, 1)
        aState.mPosition = 1;
      end
      set(aUIControls.mSliderPosition, 'Value', aState.mPosition);
    end
    RefreshPlot(aImarisApplication, aUIControls, aState, aValues, 0)
  case 13 % show 3d
    PlotThreeD(aState, aValues)
end


%-------------------------------------------------------------------------%
%% Builds the plot GUI, returns the list of the objects handles
%-------------------------------------------------------------------------%
function aUIControls = BuildGUI()

% create the window
vWindow = figure('Name', 'Imaris Intensity Profile', 'NumberTitle', 'off', ...
  'ResizeFcn', 'XTIntensityProfile(-2)', ...
  'DeleteFcn', 'XTIntensityProfile(-1)');
aUIControls.mWindow = vWindow;
%   'MenuBar', 'none', 'ToolBar', 'none'

aUIControls.mSliderPosition = uicontrol('Style', 'slider', ...
    'Min', 0, 'Max', 1, 'Value', 0,  'SliderStep', [1, 1], ...
    'Callback', 'XTIntensityProfile(-3)');
aUIControls.mSliderTime = uicontrol('Style', 'slider', ...
    'Min', 0, 'Max', 1, 'Value', 0,  'SliderStep', [1, 1], ...
    'Callback', 'XTIntensityProfile(-4)');
aUIControls.mCheckPosition = uicontrol('Style', 'radiobutton', ...
  'String', 'Position', 'Value', 1, 'Callback', 'XTIntensityProfile(-5)');
aUIControls.mCheckTime = uicontrol('Style', 'radiobutton', ...
  'String', 'Time', 'Value', 0, 'Callback', 'XTIntensityProfile(-6)');
aUIControls.mButtonRefresh = uicontrol('Style', 'pushbutton', ...
  'String', 'Refresh', 'Callback', 'XTIntensityProfile(-7)');
aUIControls.mTextMessage = uicontrol('Style', 'text', 'String', '', ...
  'ForegroundColor', [1, 0, 0], 'HorizontalAlignment', 'Left');
aUIControls.mButtonExport = uicontrol('Style', 'pushbutton', ...
  'String', 'Export', 'Callback', 'XTIntensityProfile(-8)');
aUIControls.mButtonThreeD = uicontrol('Style', 'pushbutton', ...
  'String', 'Show 3D', 'Callback', 'XTIntensityProfile(-13)');
aUIControls.mButtonPlay = uicontrol('Style', 'pushbutton', ...
  'String', 'Play', 'Callback', 'XTIntensityProfile(-9)');
aUIControls.mTextInterval = uicontrol('Style', 'text', ...
  'String', 'Time interval (sec):', 'HorizontalAlignment', 'Left');
aUIControls.mEditInterval = uicontrol('Style', 'edit', ...
  'String', '0.2', 'HorizontalAlignment', 'Left', ...
  'Callback', 'XTIntensityProfile(-10)');
aUIControls.mCheckUpdateImaris = uicontrol('Style', 'check', ...
  'String', 'Animate spots and time in Imaris', 'Value', 1, ...
  'Callback', 'XTIntensityProfile(-11)');
aUIControls.mTimer = timer('TimerFcn', 'XTIntensityProfile(-12)', ...
  'Period', 0.2, 'BusyMode', 'Drop', 'TasksToExecute', 10000, ...
  'ExecutionMode', 'fixedRate');

aUIControls.mPlotAxes = gca;


%-------------------------------------------------------------------------%
%% Resize window
%-------------------------------------------------------------------------%
function ResizeWindow(aUIControls)

vDX = 6;
vDY = 3;
vLX = 100;
vLY = 18;
vB = 25;

vWindowCoords = get(aUIControls.mWindow, 'Position');
vWindowSpace = [vWindowCoords(3:4), vWindowCoords(3:4)];

vPos = [vDX, vDY*3, vLX, vLY];
set(aUIControls.mButtonRefresh, 'Position', vPos);
vPos(1) = vPos(1) + vDX + vLX;
set(aUIControls.mButtonExport, 'Position', vPos);
vPos(1) = vPos(1) + vDX + vLX;
set(aUIControls.mButtonThreeD, 'Position', vPos);
vL = vPos(1) + vPos(3) + vDX;
vPos = [vL, vPos(2), vWindowCoords(3) - vL - vDX, vLY];
set(aUIControls.mTextMessage, 'Position', vPos);

vPos = [vDX, vPos(2) + vPos(4) + vDY, vLX, vLY];
set(aUIControls.mButtonPlay, 'Position', vPos);
vPos(1) = vPos(1) + vDX + vLX;
set(aUIControls.mTextInterval, 'Position', vPos);
vPos(1) = vPos(1) + vDX + vLX;
set(aUIControls.mEditInterval, 'Position', vPos);
vL = vPos(1) + vPos(3) + vDX;
vPos = [vL, vPos(2), vWindowCoords(3) - vL - vDX, vLY];
set(aUIControls.mCheckUpdateImaris, 'Position', vPos);

vPos = [vDX, vPos(2) + vPos(4) + vDY, vLX, vLY];
set(aUIControls.mCheckTime, 'Position', vPos);
vL = vPos(1) + vPos(3) + vDX;
vPos = [vL, vPos(2), vWindowCoords(3) - vL - vDX, vLY];
set(aUIControls.mSliderTime, 'Position', vPos);

vPos = [vDX, vPos(2) + vPos(4) + vDY, vLX, vLY];
set(aUIControls.mCheckPosition, 'Position', vPos);
vL = vPos(1) + vPos(3) + vDX;
vPos = [vL, vPos(2), vWindowCoords(3) - vL - vDX, vLY];
set(aUIControls.mSliderPosition, 'Position', vPos);

vL = vPos(2) + vPos(4) + vDY + 2*vB;
vPos = [vDX + vB, vL, vWindowCoords(3) - 3*vDX - vB, vWindowCoords(4) - vL - vDY - vB];
set(aUIControls.mPlotAxes, 'Position', vPos./vWindowSpace);


%-------------------------------------------------------------------------%
%% Refresh values
%-------------------------------------------------------------------------%
function [aValues, aState] = RefreshValues(aImarisApplication, aUIControls, aState)

aValues = [];
set(aUIControls.mTextMessage, 'String', '');

vSpots = aImarisApplication.GetFactory.ToSpots(aImarisApplication.GetSurpassSelection);
if isempty(vSpots)
  set(aUIControls.mTextMessage, 'String', 'Please select some spots! (And Refresh)');
  return
end

vCoords = vSpots.GetPositionsXYZ;
if size(vCoords, 1) < 2
  set(aUIControls.mTextMessage, 'String', 'Spots needs at least two elements!');
  return
end

vDataSet = aImarisApplication.GetDataSet;
if isempty(vDataSet)
  set(aUIControls.mTextMessage, 'String', 'Please load some dataset!');
  return
end

vNumberOfTimePoints = vDataSet.GetSizeT;
vNumberOfChannels = vDataSet.GetSizeC;

vExtMin = [vDataSet.GetExtendMinX, vDataSet.GetExtendMinY, vDataSet.GetExtendMinZ];
vExtMax = [vDataSet.GetExtendMaxX, vDataSet.GetExtendMaxY, vDataSet.GetExtendMaxZ];
vSize = [vDataSet.GetSizeX, vDataSet.GetSizeY, vDataSet.GetSizeZ];
vVoxelSize = (vExtMax - vExtMin) ./ vSize;

for vIndex = 1:3
  vCoords(:, vIndex) = min(max(vCoords(:, vIndex), vExtMin(vIndex)), vExtMax(vIndex));
end
aState.mCoords = vCoords;

vMinAll = floor((min(vCoords, [], 1) - vExtMin) ./ vVoxelSize);
vMaxAll = floor((max(vCoords, [], 1) - vExtMin) ./ vVoxelSize) + 1;
vSize = vMaxAll - vMinAll;

vEdgeSize = size(vCoords, 1) - 1;
aState.mLengths = zeros(vEdgeSize, 1);
vLength = 0;
for vEdge = 1:vEdgeSize
  vMin = floor((min(vCoords(vEdge:vEdge+1, :), [], 1) - vExtMin) ./ vVoxelSize);
  vMax = floor((max(vCoords(vEdge:vEdge+1, :), [], 1) - vExtMin) ./ vVoxelSize) + 1;
  aState.mLengths(vEdge) = max(ceil(sqrt(sum( (vMax - vMin).^2 ))), 1);
  vLength = vLength + aState.mLengths(vEdge);
end

vStart = 0;
vCoordsLin = zeros(1, vLength);

for vEdge = 1:vEdgeSize
  vLen = aState.mLengths(vEdge);
  vCoordsAll = zeros(3, vLen);
  for vIndex = 1:3
    vCoordsAll(vIndex, :) = ...
      vCoords(vEdge, vIndex) + linspace(0, 1, vLen) * ...
      (vCoords(vEdge+1, vIndex) - vCoords(vEdge, vIndex));
    vCoordsAll(vIndex, :) = ...
      (vCoordsAll(vIndex, :) - vExtMin(vIndex)) / ...
      vVoxelSize(vIndex) - vMinAll(vIndex);
  end

  vCoordsLin(vStart + (1:vLen)) = ...
    floor(vCoordsAll(3, :))*vSize(1)*vSize(2) + ...
    floor(vCoordsAll(2, :))*vSize(1) + floor(vCoordsAll(1, :)) + 1;
  vStart = vStart + vLen;
end

vMinAll = int32(vMinAll);
vSize = int32(vSize);

aValues = zeros(vLength, vNumberOfTimePoints, vNumberOfChannels);
aState.mChannelNames = cell(1, vNumberOfChannels);
aState.mChannelColors = zeros(vNumberOfChannels, 4);
for vChannel = 1:vNumberOfChannels
  for vTime = 1:vNumberOfTimePoints
    if strcmp(vDataSet.GetType, 'eTypeUInt8')
        vImage = typecast(vDataSet.GetDataSubVolumeAs1DArrayBytes( ...
          vMinAll(1), vMinAll(2), vMinAll(3), ...
          vChannel-1, vTime-1, vSize(1), vSize(2), vSize(3)), 'uint8');
    elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
        vImage = typecast(vDataSet.GetDataSubVolumeAs1DArrayShorts( ...
          vMinAll(1), vMinAll(2), vMinAll(3), ...
          vChannel-1, vTime-1, vSize(1), vSize(2), vSize(3)), 'uint16');
    elseif strcmp(vDataSet.GetType, 'eTypeFloat')
        vImage = vDataSet.GetDataSubVolumeAs1DArrayFloats( ...
          vMinAll(1), vMinAll(2), vMinAll(3), ...
          vChannel-1, vTime-1, vSize(1), vSize(2), vSize(3));
    end
    aValues(:, vTime, vChannel) = vImage(vCoordsLin);
  end
  aState.mChannelNames{vChannel} = char(vDataSet.GetChannelName(vChannel - 1));
  vRGBA = vDataSet.GetChannelColorRGBA(vChannel - 1);
  for vIndex = 1:4
    aState.mChannelColors(vChannel, vIndex) = mod(vRGBA, 256) / 255.;
    vRGBA = floor(vRGBA / 256);
  end
end

aState.mTime = aImarisApplication.GetVisibleIndexT + 1;
aState.mPosition = floor(vLength/2);

set(aUIControls.mSliderPosition, 'Min', 1);
set(aUIControls.mSliderPosition, 'Value', 1);
set(aUIControls.mSliderPosition, 'Max', vLength);
set(aUIControls.mSliderPosition, 'SliderStep', [1, 1] / max(vLength - 1, 1));
set(aUIControls.mSliderPosition, 'Value', aState.mPosition);
set(aUIControls.mSliderTime, 'Min', 1);
set(aUIControls.mSliderTime, 'Value', 1);
set(aUIControls.mSliderTime, 'Max', vNumberOfTimePoints);
set(aUIControls.mSliderTime, 'SliderStep', [1, 1] / max(vNumberOfTimePoints - 1, 1));
set(aUIControls.mSliderTime, 'Value', aState.mTime);

if isempty(aState.mSpots)
  vSpots = aImarisApplication.GetFactory.CreateSpots;
  vSpots.SetColorRGBA(255 + 255*256);
  vSpots.SetName('IntensityProfile');
  aState.mSpots = vSpots;
  vScene = aImarisApplication.GetSurpassScene;
  vScene.AddChild(vSpots, -1);
else
  vSpots = aState.mSpots;
end

vLength = vNumberOfTimePoints;
vPoints = vEdgeSize + 2;
vCoords = zeros(vPoints*vLength, 3);
for vIndex = 1:3
  for vEdge = 1:vPoints-1
    vCoords((1:vLength)*vPoints - vEdge, vIndex) = aState.mCoords(vPoints - vEdge, vIndex);
  end
  vCoords((1:vLength)*vPoints, vIndex) = aState.mCoords(1, vIndex);
end

vTimes = zeros(vPoints*vLength, 1);
for vEdge = 0:vPoints-1
  vTimes((1:vLength)*vPoints - vEdge) = 0:vLength-1;
end

vRadius = sqrt(sum(vVoxelSize.^2)) / 3;
vRadii = zeros((vEdgeSize+2)*vLength, 1);
vRadii(:) = vRadius;
vRadii((1:vLength)*vPoints) = 2*vRadius;

vTrack = [(0:vEdgeSize-1)', (1:vEdgeSize)'];

vSpots.Set(vCoords, vTimes, vRadii);
vSpots.SetTrackEdges(vTrack);

aState.mMinMax = [min(aValues(:)), max(aValues(:))];


%-------------------------------------------------------------------------%
%% Refresh plot
%-------------------------------------------------------------------------%
function RefreshPlot(aImarisApplication, aUIControls, aState, aValues, aFlag)

if isempty(aValues)
  return
end
if nargin < 5
  aFlag = 1;
end

axes(aUIControls.mPlotAxes)

newplot
hold on
if aState.mShow == 0 % position
  for vChannel = 1:size(aValues, 3)
    plot(aValues(:, aState.mTime, vChannel), ...
      'Color', aState.mChannelColors(vChannel, 1:3));
  end
  vX = aState.mPosition;
else
  for vChannel = 1:size(aValues, 3)
    plot(aValues(aState.mPosition, :, vChannel), ...
      'Color', aState.mChannelColors(vChannel, 1:3));
  end
  vX = aState.mTime;
end

vLimits = axis;
vLimits(3:4) = aState.mMinMax;
if aState.mShow == 0 % position
  vXX = 0;
  for vEdge = 1:numel(aState.mLengths)-1
    vXX = vXX + aState.mLengths(vEdge);
    plot([vXX, vXX], vLimits(3:4), 'g-')    
  end
end
plot([vX, vX], vLimits(3:4), 'k-')

vString = {'Position', 'Time'};
if aFlag
  legend(aState.mChannelNames)
end
vStringTitle = {'Time', 'a Line'};
title(['Intensity Profile along ', vStringTitle{1 + aState.mShow}])
xlabel(vString{1 + aState.mShow})
hold off

if aState.mUpdateImaris || aFlag
  vCoords = aState.mSpots.GetPositionsXYZ;
  vTimes = aState.mSpots.GetIndicesT;
  vRadii = aState.mSpots.GetRadiiXYZ;
  vStart = 1;
  for vEdge = 1:numel(aState.mLengths)
    vLen = aState.mLengths(vEdge);
    if aState.mPosition < vStart + vLen
      vCoords1 = vCoords(vEdge, :);
      vCoords2 = vCoords(vEdge + 1, :);
      vCoords(aState.mTime*(numel(aState.mLengths)+2), :) = ...
        vCoords1 + (vCoords2 - vCoords1) * ...
        (aState.mPosition-vStart+1)/(max(vLen, 1));
      break
    end
    vStart = vStart + vLen;
  end

  vTrack = aState.mSpots.GetTrackEdges;
  aState.mSpots.Set(vCoords, vTimes, vRadii(:,1));
  aState.mSpots.SetRadiiXYZ(vRadii);
  aState.mSpots.SetTrackEdges(vTrack);

  aImarisApplication.SetVisibleIndexT(aState.mTime - 1);
end

vTimeString = {'Play', 'Stop'};
set(aUIControls.mButtonPlay, 'String', ...
  [vTimeString{1+aState.mPlaying}, ' (', vString{2-aState.mShow}, ')'])


%-------------------------------------------------------------------------%
%% Refresh plot
%-------------------------------------------------------------------------%
function PlotThreeD(aState, aValues)

figure
surf(aValues(:, :, 1))
% colormap hot
shading interp
colorbar
title(['Intensity Profile of ', aState.mChannelNames{1}])
ylabel('Position')
xlabel('Time')


%-------------------------------------------------------------------------%
%% Export
%-------------------------------------------------------------------------%
function ExportValues(aImarisApplication, aState, aValues)

[vFile, vPath] = uiputfile({'*.txt', 'Text-file (*.txt)'; ...
  '*.mat', 'Matlab-file (*.mat)'; ...
  '*.csv', 'Comma separated values (*.csv)'; '*.*', 'All files'}, ...
  'XTIntensityProfile Export', 'Data.txt');
if isempty(vFile) || isempty(vPath)
  return
end

[vPath, vFile, vExt] = fileparts(fullfile(vPath, vFile));
vFileName = fullfile(vPath, [vFile, vExt]);
vExt = lower(vExt);

vCoords = aState.mSpots.GetPositionsXYZ;
if ~isequal(vExt, '.mat')
  vItem = ' ';
  if isequal(vExt, '.csv')
    vItem = ',';
  end
  vType = '%d';
  if strcmp(aImarisApplication.GetDataSet.GetType, 'eTypeFloat')
    vType = '%f';
  end

  vFile = fopen(vFileName, 'wt');
  fprintf(vFile, 'Intensity profile of %s between \n', ...
    char(aImarisApplication.GetCurrentFileName));
  
  for vEdge = 1:numel(aState.mLengths)
    fprintf(vFile, ['\n [%f, %f, %f] and [%f, %f, %f] (%d points)', ...
      vItem], vCoords(vEdge, 1), vCoords(vEdge, 2), vCoords(vEdge, 3), ...
      vCoords(vEdge+1, 1), vCoords(vEdge+1, 2), vCoords(vEdge+1, 3), ...
      aState.mLengths(vEdge));
  end
  fprintf(vFile, '\n\n');
  for vChannel = 1:size(aValues, 3)
    fprintf(vFile, [aState.mChannelNames{vChannel}, vItem, '\n']);
    for vTime = 1:size(aValues, 2)
      vStart = 0;
      for vEdge = 1:numel(aState.mLengths)
        vLen = aState.mLengths(vEdge);
        for vIndex = vStart + (1:vLen-1)
          fprintf(vFile, [vType, vItem], aValues(vIndex, vTime, vChannel));
        end
        vStart = vStart + vLen;
        fprintf(vFile, [vType, ',' vItem], aValues(vStart, vTime, vChannel));
      end
      fprintf(vFile, '\n');
    end
    fprintf(vFile, '\n');
  end
  fclose(vFile);
else
  eval('FileName = char(aImarisApplication.GetCurrentFileName);')
  eval('Coordinates = vCoords(1:numel(aState.mLengths)+1, :);')
  eval('NumbeOfIntermediatePoints = aState.mLengths;')
  eval('Intensities = aValues;')
  save(vFileName, 'FileName', 'Coordinates', ...
    'NumbeOfIntermediatePoints', 'Intensities')
end

